@@ -27,6 +27,7 @@
 #include <linux/mutex.h>
 #include <linux/proc_fs.h>
 #include <linux/seq_file.h>
+#include <linux/slab.h>
 #ifdef	PROTOCOL_DEBUG
 #include <linux/ctype.h>
 #endif
@@ -60,8 +61,8 @@ static DEF_PARM(uint, command_queue_length, 1500, 0444,
 static DEF_PARM(uint, poll_timeout, 1000, 0644,
 		"Timeout (in jiffies) waiting for units to reply");
 static DEF_PARM_BOOL(rx_tasklet, 0, 0644, "Use receive tasklets");
-static DEF_PARM_BOOL(dahdi_autoreg, 1, 0644,
-		     "Register devices automatically (1) or not (0)");
+static DEF_PARM_BOOL(dahdi_autoreg, 0, 0444,
+		     "Register devices automatically (1) or not (0). UNUSED.");
 
 #ifdef	CONFIG_PROC_FS
 static const struct file_operations xbus_read_proc_ops;
@@ -79,8 +80,14 @@ static struct proc_dir_entry *proc_xbuses;
 
 static struct xbus_desc {
 	xbus_t *xbus;
+	int shutting_down;
 } xbuses_array[MAX_BUSES];
 
+static int xbus_is_shutting_down(int num)
+{
+	return xbuses_array[num].shutting_down;
+}
+
 static xbus_t *xbus_byhwid(const char *hwid)
 {
 	int i;
@@ -148,6 +155,8 @@ static void init_xbus(uint num, xbus_t *xbus)
 	BUG_ON(num >= ARRAY_SIZE(xbuses_array));
 	desc = &xbuses_array[num];
 	desc->xbus = xbus;
+	if (xbus)
+		desc->shutting_down = 0;
 }
 
 xbus_t *xbus_num(uint num)
@@ -187,9 +196,12 @@ static void finalize_xbuses_array(void)
 static void xbus_destroy(struct kref *kref)
 {
 	xbus_t *xbus;
+	int num;
 
 	xbus = kref_to_xbus(kref);
-	XBUS_NOTICE(xbus, "%s\n", __func__);
+	XBUS_DBG(DEVICES, xbus, "%s\n", __func__);
+	num = xbus->num;
+	xbuses_array[num].shutting_down = 1;
 	xbus_sysfs_remove(xbus);
 }
 
@@ -198,6 +210,11 @@ xbus_t *get_xbus(const char *msg, uint num)
 	unsigned long flags;
 	xbus_t *xbus;
 
+	if (xbus_is_shutting_down(num)) {
+		DBG(DEVICES, "%s(%s): XBUS-%d: shutting down\n", __func__,
+				msg, num);
+		return NULL;
+	}
 	spin_lock_irqsave(&xbuses_lock, flags);
 	xbus = xbus_num(num);
 	if (xbus != NULL) {
@@ -211,6 +228,13 @@ xbus_t *get_xbus(const char *msg, uint num)
 
 void put_xbus(const char *msg, xbus_t *xbus)
 {
+	if (xbus_is_shutting_down(xbus->num)) {
+		if (!refcount_xbus(xbus)) {
+			DBG(DEVICES, "%s(%s): XBUS-%d: shutting down\n",
+					__func__, msg, xbus->num);
+			return;
+		}
+	}
 	XBUS_DBG(DEVICES, xbus, "%s: refcount_xbus=%d\n", msg,
 		 refcount_xbus(xbus));
 	kref_put(&xbus->kref, xbus_destroy);
@@ -220,7 +244,7 @@ int refcount_xbus(xbus_t *xbus)
 {
 	struct kref *kref = &xbus->kref;
 
-	return atomic_read(&kref->refcount);
+	return refcount_read(&kref->refcount);
 }
 
 /*------------------------- Frame  Handling ------------------------*/
@@ -560,6 +584,14 @@ static void receive_tasklet_func(unsigned long data)
 void xbus_receive_xframe(xbus_t *xbus, xframe_t *xframe)
 {
 	BUG_ON(!xbus);
+	if (xbus_is_shutting_down(xbus->num)) {
+		static int rate_limit;
+
+		if ((rate_limit++ % 1000) == 0)
+			XBUS_NOTICE(xbus, "%s: during shutdown (%d)\n",
+					__func__, rate_limit);
+		return;
+	}
 	if (rx_tasklet) {
 		xframe_enqueue_recv(xbus, xframe);
 	} else {
@@ -655,10 +687,56 @@ int xbus_xpd_unbind(xbus_t *xbus, xpd_t *xpd)
 	return 0;
 }
 
-static int new_card(xbus_t *xbus, int unit, __u8 type, __u8 subtype,
-		    __u8 numchips, __u8 ports_per_chip, __u8 ports,
-		    __u8 port_dir)
+static xpd_type_t xpd_hw2xpd_type(const struct unit_descriptor *unit_descriptor)
 {
+	xpd_type_t xpd_type;
+
+	switch (unit_descriptor->type) {
+	case 1:
+	case 6:
+		xpd_type = XPD_TYPE_FXS;
+		break;
+	case 2:
+		xpd_type = XPD_TYPE_FXO;
+		break;
+	case 3:
+		xpd_type = XPD_TYPE_BRI;
+		break;
+	case 4:
+		xpd_type = XPD_TYPE_PRI;
+		break;
+	case 5:
+		xpd_type = XPD_TYPE_ECHO;
+		break;
+	case 7:
+		xpd_type = XPD_TYPE_NOMODULE;
+		break;
+	default:
+		NOTICE("WARNING: xpd hw type is: %d\n", unit_descriptor->type);
+		xpd_type = XPD_TYPE_NOMODULE;
+		break;
+	}
+	return xpd_type;
+}
+
+int subunits_of_xpd(const struct unit_descriptor* unit_descriptor,
+		const xproto_table_t *proto_table) {
+	int ports = unit_descriptor->ports_per_chip * unit_descriptor->numchips;
+
+	return
+		(ports + proto_table->ports_per_subunit - 1)
+		/ proto_table->ports_per_subunit;
+}
+
+static int new_card(xbus_t *xbus, const struct unit_descriptor *unit_descriptor)
+{
+	int unit = unit_descriptor->addr.unit;
+	xpd_type_t xpd_type;
+	__u8 hw_type;
+	__u8 numchips;
+	__u8 ports_per_chip;
+	__u8 ports;
+	__u8 port_dir;
 	const xproto_table_t *proto_table;
 	int i;
 	int subunits;
@@ -666,12 +744,19 @@ static int new_card(xbus_t *xbus, int unit, __u8 type, __u8 subtype,
 	int remaining_ports;
 	const struct echoops *echoops;
 
-	proto_table = xproto_get(type);
+	/* Translate parameters from "unit_descriptor" */
+	hw_type = unit_descriptor->type;
+	numchips = unit_descriptor->numchips;
+	ports_per_chip = unit_descriptor->ports_per_chip;
+	port_dir = unit_descriptor->port_dir;
+	ports = unit_descriptor->ports_per_chip * unit_descriptor->numchips;
+	xpd_type = xpd_hw2xpd_type(unit_descriptor);
+	proto_table = xproto_get(xpd_type);
 	if (!proto_table) {
 		XBUS_NOTICE(xbus,
-			"CARD %d: missing protocol table for type %d. "
+			"CARD %d: missing protocol table for xpd_type %d. "
 			"Ignored.\n",
-			unit, type);
+			unit, xpd_type);
 		return -EINVAL;
 	}
 	echoops = proto_table->echoops;
@@ -679,26 +764,24 @@ static int new_card(xbus_t *xbus, int unit, __u8 type, __u8 subtype,
 		XBUS_INFO(xbus, "Detected ECHO Canceler (%d)\n", unit);
 		if (ECHOOPS(xbus)) {
 			XBUS_NOTICE(xbus,
-				"CARD %d: tryies to define echoops (type %d) "
+				"CARD %d: tryies to define echoops (xpd_type %d) "
 				"but we already have one. Ignored.\n",
-				unit, type);
+				unit, xpd_type);
 			return -EINVAL;
 		}
 		xbus->echo_state.echoops = echoops;
 		xbus->echo_state.xpd_idx = XPD_IDX(unit, 0);
 	}
 	remaining_ports = ports;
-	subunits =
-	    (ports + proto_table->ports_per_subunit -
-	     1) / proto_table->ports_per_subunit;
+	subunits = subunits_of_xpd(unit_descriptor, proto_table);
 	XBUS_DBG(DEVICES, xbus,
-		"CARD %d type=%d.%d ports=%d (%dx%d), "
+		"CARD %d xpd_type=%d/hw_type=%d ports=%d (%dx%d), "
 		"%d subunits, port-dir=0x%02X\n",
-		unit, type, subtype, ports, numchips, ports_per_chip, subunits,
+		unit, xpd_type, hw_type, ports, numchips, ports_per_chip, subunits,
 		port_dir);
-	if (type == XPD_TYPE_PRI || type == XPD_TYPE_BRI)
+	if (xpd_type == XPD_TYPE_PRI || xpd_type == XPD_TYPE_BRI)
 		xbus->quirks.has_digital_span = 1;
-	if (type == XPD_TYPE_FXO)
+	if (xpd_type == XPD_TYPE_FXO)
 		xbus->quirks.has_fxo = 1;
 	xbus->worker.num_units += subunits - 1;
 	for (i = 0; i < subunits; i++) {
@@ -722,11 +805,10 @@ static int new_card(xbus_t *xbus, int unit, __u8 type, __u8 subtype,
 			goto out;
 		}
 		XBUS_DBG(DEVICES, xbus,
-			 "Creating XPD=%d%d type=%d.%d (%d ports)\n", unit, i,
-			 type, subtype, subunit_ports);
+			 "Creating XPD=%d%d xpd_type=%d.%d hw_type=%d (%d ports)\n", unit, i,
+			 xpd_type, unit_descriptor->subtype, hw_type, subunit_ports);
 		ret =
-		    create_xpd(xbus, proto_table, unit, i, type, subtype,
-			       subunits, subunit_ports, port_dir);
+		    create_xpd(xbus, proto_table, unit_descriptor, unit, i, xpd_type);
 		if (ret < 0) {
 			XBUS_ERR(xbus, "Creation of XPD=%d%d failed %d\n", unit,
 				 i, ret);
@@ -953,10 +1035,14 @@ int xbus_register_dahdi_device(xbus_t *xbus)
 		goto err;
 	}
 	if (xbus_is_registered(xbus)) {
-		XBUS_ERR(xbus, "Already registered to DAHDI\n");
-		WARN_ON(1);
-		ret = -EINVAL;
-		goto err;
+		/*
+		 * Ignore duplicate registrations (from dahdi_registration)
+		 * Until we completely migrate to dahdi_autoreg=1 and
+		 * hotplug-based span-assignments
+		 */
+		XBUS_DBG(DEVICES, xbus, "Already registered to DAHDI\n");
+		ret = 0;
+		goto out;
 	}
 	xbus->ddev = dahdi_create_device();
 	if (!xbus->ddev) {
@@ -1030,26 +1116,34 @@ void xbus_unregister_dahdi_device(xbus_t *xbus)
 	int i;
 	int ret;
 
-	XBUS_NOTICE(xbus, "%s\n", __func__);
+	XBUS_DBG(DEVICES, xbus, "%s\n", __func__);
 	ret = mutex_lock_interruptible(&dahdi_registration_mutex);
 	if (ret < 0) {
 		XBUS_ERR(xbus, "dahdi_registration_mutex already taken\n");
 		return;
 	}
+	if (!xbus_is_registered(xbus)) {
+		/*
+		 * Ignore duplicate unregistrations
+		 */
+		XBUS_DBG(DEVICES, xbus, "Already unregistered to DAHDI\n");
+		goto err;
+	}
 	for (i = 0; i < MAX_XPDS; i++) {
 		xpd_t *xpd = xpd_of(xbus, i);
 		xpd_dahdi_preunregister(xpd);
 	}
 	if (xbus->ddev) {
 		dahdi_unregister_device(xbus->ddev);
-		XBUS_NOTICE(xbus, "%s: finished dahdi_unregister_device()\n",
-			    __func__);
+		XBUS_DBG(DEVICES, xbus,
+			"%s: finished dahdi_unregister_device()\n", __func__);
 		xbus_free_ddev(xbus);
 	}
 	for (i = 0; i < MAX_XPDS; i++) {
 		xpd_t *xpd = xpd_of(xbus, i);
 		xpd_dahdi_postunregister(xpd);
 	}
+err:
 	mutex_unlock(&dahdi_registration_mutex);
 }
 
@@ -1086,11 +1180,7 @@ void xbus_populate(void *data)
 		BUG_ON(card_desc->magic != CARD_DESC_MAGIC);
 		/* Release/Reacquire locks around blocking calls */
 		spin_unlock_irqrestore(&xbus->worker.worker_lock, flags);
-		ret =
-		    new_card(xbus, card_desc->xpd_addr.unit, card_desc->type,
-			     card_desc->subtype, card_desc->numchips,
-			     card_desc->ports_per_chip, card_desc->ports,
-			     card_desc->port_dir);
+		ret = new_card(xbus, &card_desc->unit_descriptor);
 		spin_lock_irqsave(&xbus->worker.worker_lock, flags);
 		KZFREE(card_desc);
 		if (ret)
@@ -1118,7 +1208,7 @@ void xbus_populate(void *data)
 	 */
 	xbus_request_sync(xbus, SYNC_MODE_PLL);
 	elect_syncer("xbus_populate(end)");	/* FIXME: try to do it later */
-	if (dahdi_autoreg)
+	if (!dahdi_get_auto_assign_spans())
 		xbus_register_dahdi_device(xbus);
 out:
 	XBUS_DBG(DEVICES, xbus, "Leaving\n");
@@ -1176,8 +1266,7 @@ static void worker_reset(xbus_t *xbus)
 	name = (xbus) ? xbus->busname : "detached";
 	DBG(DEVICES, "%s\n", name);
 	if (!worker->xpds_init_done) {
-		NOTICE("%s: worker(%s)->xpds_init_done=%d\n", __func__, name,
-		       worker->xpds_init_done);
+		XBUS_NOTICE(xbus, "XPDS initialization was not finished\n");
 	}
 	spin_lock_irqsave(&worker->worker_lock, flags);
 	list_for_each_safe(card, next_card, &worker->card_list) {
@@ -1195,13 +1284,15 @@ static void worker_reset(xbus_t *xbus)
 	spin_unlock_irqrestore(&worker->worker_lock, flags);
 }
 
+/*
+ * Called only after worker_reset(xbus)
+ */
 static void worker_destroy(xbus_t *xbus)
 {
 	struct xbus_workqueue *worker;
 
 	BUG_ON(!xbus);
 	worker = &xbus->worker;
-	worker_reset(xbus);
 	XBUS_DBG(DEVICES, xbus, "Waiting for worker to finish...\n");
 	down(&worker->running_initialization);
 	XBUS_DBG(DEVICES, xbus, "Waiting for worker to finish -- done\n");
@@ -1254,6 +1345,7 @@ static int worker_run(xbus_t *xbus)
 	}
 	return 1;
 err:
+	worker_reset(xbus);
 	worker_destroy(xbus);
 	return 0;
 }
@@ -1421,6 +1513,7 @@ void xbus_disconnect(xbus_t *xbus)
 	del_timer_sync(&xbus->command_timer);
 	transportops_put(xbus);
 	transport_destroy(xbus);
+	/* worker_reset(xbus) was called in xbus_deactivate(xbus) */
 	worker_destroy(xbus);
 	XBUS_DBG(DEVICES, xbus, "Deactivated refcount_xbus=%d\n",
 		 refcount_xbus(xbus));
@@ -1470,6 +1563,7 @@ void xbus_free(xbus_t *xbus)
 	num = xbus->num;
 	BUG_ON(!xbuses_array[num].xbus);
 	BUG_ON(xbus != xbuses_array[num].xbus);
+	init_xbus(num, NULL);
 	spin_unlock_irqrestore(&xbuses_lock, flags);
 #ifdef CONFIG_PROC_FS
 	if (xbus->proc_xbus_dir) {
@@ -1496,7 +1590,6 @@ void xbus_free(xbus_t *xbus)
 #endif
 	spin_lock_irqsave(&xbuses_lock, flags);
 	XBUS_DBG(DEVICES, xbus, "Going to free...\n");
-	init_xbus(num, NULL);
 	spin_unlock_irqrestore(&xbuses_lock, flags);
 	KZFREE(xbus);
 }
@@ -1627,10 +1720,6 @@ int waitfor_xpds(xbus_t *xbus, char *buf)
 	 * FIXME: worker is created before ?????
 	 * So by now it exists and initialized.
 	 */
-	/* until end of waitfor_xpds_show(): */
-	xbus = get_xbus(__func__, xbus->num);
-	if (!xbus)
-		return -ENODEV;
 	worker = &xbus->worker;
 	BUG_ON(!worker);
 	if (!worker->wq) {
@@ -1669,7 +1758,6 @@ int waitfor_xpds(xbus_t *xbus, char *buf)
 		spin_unlock_irqrestore(&xbus->lock, flags);
 	}
 out:
-	put_xbus(__func__, xbus);	/* from start of waitfor_xpds_show() */
 	return len;
 }
 
@@ -1977,6 +2065,10 @@ int __init xbus_core_init(void)
 {
 	int ret = 0;
 
+	if (dahdi_autoreg == 1) {
+		NOTICE("WARNING: The dahdi_autoreg parameter is deprecated " \
+			"-- just set dahdi.auto_assign_spans=0\n");
+	}
 	initialize_xbuses_array();
 #ifdef PROTOCOL_DEBUG
 	INFO("FEATURE: with PROTOCOL_DEBUG\n");
